clear all; close all;
%% Project assumptions - input data

TD.I.ModM = 0;  % Mod TEmk1
TD.I.ModK = 1;  % Mod TEmk1    
TD.I.ResonatorRadiusR0 = 7.5; %[mm]
TD.I.HarmonicNumber = 1;
% [degree] angle between resonator parts
TD.I.ResonatorAngle = [2.3;0.0;3.00];  
% [mm] resonator length, input tapper, middle part, output tapper
TD.I.ResonatorLength = [30.0; 70.9; 30.0];
TD.I.StepZ =0.05; % Z axis step [mm]  must be smaller than: c/f * (1/20)

%% Constants
C = PhysicalConstants;
%% Time Dependent Approach
TD.Movie = 0;  % 0 - dont export movie, 1 - will export movie as bmp images

% INPUT variables
TD.I.TimeSpan = (0:0.05:0.1); %[ns] (simulation time vector start:time step: stop)
TD.I.NumberOfElectrons = 24;
TD.I.Alpha = 1.4;  % pitch factor
TD.I.Ucathode = 12; % [kV] cathode voltage
TD.I.GyrotronOutputPower = 1; % [kW] assumed output power

TD.I.MagneticFieldB =  0.87878 ; % magnetic field in the resonator [T]
TD.I.Icathode = 0.27;
TD.I.ElectronBeamRadius = 3.594643828953271; % if 0 is input function will calculate beam radius
TD.I.CharacteristicValue = 3.831705970641104; 
TD.I.Frequency = 24.438944314915210;    % optimal frequency for this working point
TD.I.Qtotal = 9.738828104904780e+02;    % optimal Q for this working point - total includes ohmic loos
TD.I.mnSign = 1;  % direction of mode rotation

%% Cavity dimmension calculation - radius vector - begin
% z axis
ResonatorTotalLength = sum(TD.I.ResonatorLength);
ResonatorCentralLength = TD.I.ResonatorLength(2);
ResonatorSliceCount = ResonatorTotalLength / TD.I.StepZ;
TD.I.RFfieldAxisZ = TD.I.StepZ*(0:ResonatorSliceCount)';
% radius vector
ResonatorAngleTanges = tan(TD.I.ResonatorAngle*pi/180);
R1 = TD.I.ResonatorRadiusR0 + (TD.I.RFfieldAxisZ-TD.I.ResonatorLength(1))*ResonatorAngleTanges(1);
R2 = TD.I.ResonatorRadiusR0 + (TD.I.RFfieldAxisZ-TD.I.ResonatorLength(1))*ResonatorAngleTanges(2);
R3 = TD.I.ResonatorRadiusR0 + TD.I.ResonatorLength(2)*ResonatorAngleTanges(2) + (TD.I.RFfieldAxisZ-(TD.I.ResonatorLength(1)+TD.I.ResonatorLength(2)))*ResonatorAngleTanges(3);
n1 = fix(TD.I.ResonatorLength(1)/TD.I.StepZ)+1;
n2 = fix((TD.I.ResonatorLength(1)+TD.I.ResonatorLength(2))/TD.I.StepZ)+1;
TD.I.ResonatorRadius = [R1(1:n1);R2(n1+1:n2);R3(n2+1:end)];
%% Cavity dimmension calculation - end

%% Initial conditions for RF field
TD.I.InitialField = 0.1*exp(1i*2*pi*rand(length(TD.I.RFfieldAxisZ),1));  % Random complex numbers as initial RF field profile

INPUT = TD.I;


TD.O.Ucathode = TD.I.Ucathode;
INPUT.Ucathode = TD.I.Ucathode;
INPUT.TimeSpan = TD.I.TimeSpan;    
   
OUTPUT = TimeDependentApproachFunction(INPUT, C);
INPUT.InitialField = OUTPUT.RFfieldInTime(:,end);
TD.O.USTEP(1) = OUTPUT;

TD.O.RFfieldInTime = []; 
TD.O.GyrotronOutputPower = 0; 
TD.O.RFfieldInTime = [TD.O.RFfieldInTime, TD.O.USTEP(1).RFfieldInTime()]; 
TD.O.GyrotronOutputPower = [TD.O.GyrotronOutputPower; TD.O.USTEP(1).GyrotronOutputPower()];
TD.O.TimeSpan = TD.I.TimeSpan;


%% Zapis wynikow do pliku
Folder = 'wyniki/';

hash = datestr(now,30);
FolderName=sprintf('%s%s', Folder, hash);
mkdir(FolderName);
fileParameters = sprintf('%s/%s', FolderName, 'parameters.txt');
fileOutPowerFig = sprintf('%s/%s', FolderName, 'OutputPower');
fileRFfieldFig = sprintf('%s/%s', FolderName, 'RFfield');
fileResults = sprintf('%s/%s', FolderName, 'results.mat');

fileHandle = fopen ( fileParameters , 'wt' );

dane=sprintf('\n=============================== SINGLE MODE TIME DEPENDENT PART %s ===============================', hash);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('TE: M=%d K=%d P=%d HarmonicNumber=%d ResonatorRadiusR0=%.15g [mm]',TD.I.ModM, TD.I.ModK, 1, TD.I.HarmonicNumber, TD.I.ResonatorRadiusR0);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('ResonatorAngle = %.15g; %.15g; %.15g [deg]\nResonatorLength = %.15g; %.15g; %.15g StepZ = %.15g [mm]',TD.I.ResonatorAngle, TD.I.ResonatorLength, TD.I.StepZ);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Frequency= %.10g [GHz] Qtotal = %.10g [a.u.]', TD.I.Frequency, TD.I.Qtotal);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('CharacteristicValue = %.15g',TD.I.CharacteristicValue);disp(dane); fprintf ( fileHandle , '%s\n' , dane );

dane=sprintf('NumberOfElectrons = %.15g Alpha = %.15g Ucathode = %.15g [kV] MagneticFieldB = %.15g [T]',TD.I.NumberOfElectrons, TD.I.Alpha, TD.I.Ucathode, TD.I.MagneticFieldB);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('ElectronBeamRadius = %.15g [mm], ModSign = %d', TD.I.ElectronBeamRadius, TD.I.mnSign);disp(dane); fprintf ( fileHandle , '%s\n' , dane );


dane=sprintf('TimeSpan = %.15g:%.12g:%.15g [ns]',TD.I.TimeSpan(1), (TD.I.TimeSpan(2)- TD.I.TimeSpan(1)), TD.I.TimeSpan(end));disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Ucathode = %.15g [kV]',TD.I.Ucathode);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Icathode = %.4f [A], Efficiency = %.2f, ElectronBeamRadius = %.4f', TD.I.Icathode, TD.O.USTEP.GyrotronEfficiency(end), TD.O.USTEP.ElectronBeamRadius);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Solver = %s ', TD.O.USTEP(1).Solver);disp(dane); fprintf ( fileHandle , '%s\n' , dane );

fclose(fileHandle);

save(fileResults, 'TD');

%% Plot output power figure

figHandle = figure();
plot(TD.O.TimeSpan,TD.O.GyrotronOutputPower);hold on;
plot(TD.O.TimeSpan,TD.O.Ucathode); hold off;
str=sprintf('U=%.2f [kV]',TD.O.Ucathode);
legend('Output Power [kW]',str, 'Location', 'NorthWest');
set(gcf(),'Name','Output Power [kW]');
xlabel('Time [ns]'); ylabel('Output Power [kW]');
title(sprintf('Hash: %s',hash));
savefig(figHandle, fileOutPowerFig);

%% RF field plotting

TimeMomentsToDisplay = [0, 0.5, 1, 5, 10, 20, 30, 50, 100, 150];
legendStr = '';
for i=1:1:length(TimeMomentsToDisplay)
    
   if(~isempty(find(TD.O.TimeSpan == TimeMomentsToDisplay(i))))
        ind(i)= find(TD.O.TimeSpan == TimeMomentsToDisplay(i));     
        legendStr{i} = sprintf('%.1f [ns]', TimeMomentsToDisplay(i));
   end
end
   
figHandle = figure();set(gcf(),'Name','f(z,t)');


for k=1:size(ind,2)
    if size(ind(k))
        plot(TD.I.RFfieldAxisZ,abs(TD.O.RFfieldInTime(:,ind(k))));hold on;   
    end
end
hold off; %legend('0.0[ns]','0.5[ns]','1.0[ns]','5.0[ns]','10.0[ns]','20.0[ns]','30.0[ns]','50 [ns]','100 [ns]','200 [ns]');
legend(legendStr);

title(sprintf('Hash: %s',hash));
savefig(figHandle, fileRFfieldFig);

%% Movie with rising E
if(TD.Movie == 1)
    mkdir(sprintf('%s/%s', FolderName, 'movie'));
 
    for k=1:size(abs(TD.O.RFfieldInTime(:,:)),2)
        FigHandle = figure();
        
        plot(TD.I.RFfieldAxisZ,abs(TD.O.RFfieldInTime(:,k)));
        xlim([0 max(TD.I.RFfieldAxisZ)]);
        ylim([0 max(max(abs(TD.O.RFfieldInTime(:,:))))*1.1]);
        xlabel('Z axis [mm]', 'fontsize', 12);
               
        ylabel(sprintf('Rf Field [a.u], Pout=%0.1f [kW]' , TD.O.GyrotronOutputPower(k)), 'fontsize', 12);        
        title(sprintf('Hash: %s, czas: %.2f [ns], Uc=%0.1f [kV]' ,hash, TD.I.TimeSpan(k), TD.O.Ucathode));
        
        %inset
        InsetHandle = figure();
        plot(TD.O.TimeSpan(k),TD.O.GyrotronOutputPower(k),'r.','MarkerSize',20);hold on;
        plot(TD.O.TimeSpan,TD.O.GyrotronOutputPower);set(gca,'FontSize',6); hold off;
        xlabel('Time [ns]', 'fontsize', 6); ylabel('Output Power [kW]', 'fontsize', 6);
        
        [~, NewAxHandle, NewFigHandle]=inset(FigHandle,InsetHandle);
        
        set(gcf, 'paperunits', 'centimeters', 'paperposition', [0 0 19.5 11]);
        set(NewFigHandle, 'InvertHardcopy', 'off');
        
        saveas(NewFigHandle, sprintf('%s/movie/%06.2f.bmp',  FolderName, TD.I.TimeSpan(k)), 'bmp');
        
        %saveas(NewFigHandle, sprintf('test.bmp',  FolderName, TD.I.TimeSpan(k)), 'bmp');
        close(FigHandle); close(InsetHandle); close(NewFigHandle); 
        
    end
end
