function [ OUT ] = TimeDependentApproachFunction( IN, C )

% IN - input
% L - local
solver = 'ode45'; % ode4 possible

%% Time Dependent Approach

L.CharacteristicValue = bessel_deriv_zeros(IN.ModM,IN.ModK);

OmegaCutOff = C.SpeedOfLight*L.CharacteristicValue/IN.ResonatorRadiusR0;
OmegaZ = C.SpeedOfLight*L.CharacteristicValue./IN.ResonatorRadius;

InitialFieldNormalized = IN.InitialField/max(abs(IN.InitialField));

OmegaResonator = 2*pi*IN.Frequency;
% Relativistic factor 
% 1+ E0/me*c^2 Ucathode/Energia spoczynkowa elektronu w
% keV - jednostka zmienia wartosc!!
% L.GammaElectron = 1 + IN.Ucathode/511;
L.GammaElectron = 1 + IN.Ucathode/((C.ElectronMass*(C.SpeedOfLight)^2)/C.ElectronCharge);


% Normalized electron perpendicular and parallel velocities
L.BetaPerpendicular = ( (sqrt((L.GammaElectron^2 - 1)/(1+IN.Alpha^2))) * IN.Alpha ) / L.GammaElectron;
L.BetaParallel = sqrt(1 - 1/(L.GammaElectron^2) - L.BetaPerpendicular^2);

% Electron cyclotron frequency 
% omega = e*B_0/gamma/m_e
% OmegaCyclotron = 2*pi*28*Cav.B/L.GammaElectron;
%  28 ~= (Phy.e/Phy.me/2/pi)*1e-9
OmegaCyclotron = (C.ElectronCharge/C.ElectronMass)*IN.MagneticFieldB/L.GammaElectron;

% zeta axis
L.AxisZeta = L.BetaPerpendicular^2*OmegaCyclotron*IN.RFfieldAxisZ/(2*L.BetaParallel*C.SpeedOfLight);



% Electron Beam Radius

L.ElectronBeamRadius = bessel_deriv_zeros(IN.ModM+IN.mnSign*IN.HarmonicNumber,1)*(C.SpeedOfLight/IN.Frequency)/(2*pi);


if(IN.ElectronBeamRadius >0 )
    L.ElectronBeamRadius = IN.ElectronBeamRadius;
    disp('TD: Fixed Electron Beam Radius Given.');
end


L.delta = (8*L.BetaParallel^2*L.BetaPerpendicular^(-4)/OmegaCyclotron)*(OmegaCutOff - OmegaZ);
% K for last z
L.kZend = (2*C.SpeedOfLight*L.BetaParallel*L.BetaPerpendicular^(-2)/OmegaCyclotron)*vsqrt(L.CharacteristicValue^2/IN.ResonatorRadiusR0^2 - L.CharacteristicValue^2/IN.ResonatorRadius(end)^2);


   Icathode = IN.Icathode;
 
   IcathodeNormalized = Icathode*(3.76e-3)*L.BetaParallel*L.BetaPerpendicular^(-6)*...
    besselj(IN.ModM+IN.mnSign*IN.HarmonicNumber,L.CharacteristicValue*...
    L.ElectronBeamRadius/IN.ResonatorRadiusR0)^2/(4*L.GammaElectron*...
    besselj(IN.ModM,L.CharacteristicValue)^2*(L.CharacteristicValue^2-IN.ModM^2));

L.FrequencyMismatch = (2/L.BetaPerpendicular^2)*((OmegaCutOff-IN.HarmonicNumber*OmegaCyclotron)/OmegaCyclotron);
% Electron efficiency
L.ElectronEfficiency = (1+L.GammaElectron)*IN.Alpha^2/(2*L.GammaElectron*(1+IN.Alpha^2));


Field = zeros(size(InitialFieldNormalized,1),length(IN.TimeSpan));

GyrotronEfficiency = zeros(length(IN.TimeSpan)-1, 1);
GyrotronOutputPower = zeros(length(IN.TimeSpan)-1, 1);

step = 1;
Field(:,step) = IN.InitialField;
Theta = 2*pi*(0:IN.NumberOfElectrons-1)/IN.NumberOfElectrons;
P0 = exp(1i*Theta); % warunki początkowe - rozwiązujemy równanie dla róznych Theta od 0 do 2*pi z wymyślonym krokiem zależącym od liczby elektronów


AxisZeta = L.AxisZeta;
delta = L.delta;
kZend = L.kZend;
HarmonicNumber = IN.HarmonicNumber;
NumberOfElectrons = IN.NumberOfElectrons;
FrequencyMismatch = L.FrequencyMismatch;

 F = 1; % F not used in time dependent
%% Loop

str=sprintf('--------------- STARTING TIME EVOLUTION Uc= %0.1f ---------------', IN.Ucathode);disp(str);
for t=IN.TimeSpan(1:end-1)
    PallOneStep = zeros(size(InitialFieldNormalized,1),IN.NumberOfElectrons);
    field = Field(:,step);
    fmax = max(abs(field));
    
    field_norm = field;
     
    timerVal = tic;
    S1 = griddedInterpolant(AxisZeta,field_norm,'spline');
    
    parfor k=1:NumberOfElectrons 
        if strcmp (solver, 'ode45')
            [~, Pnew] = ode45(@(ZetaNew, Pnew) OdeEqElectronTransverseMomentum(ZetaNew, Pnew, real(field_norm), imag(field_norm), AxisZeta, FrequencyMismatch, HarmonicNumber, F, S1) , AxisZeta , P0(k));
        else
            [~, Pnew] = ode4(AxisZeta , P0(k),  real(field_norm), imag(field_norm), AxisZeta, FrequencyMismatch, HarmonicNumber, F, S1);    
        end
        PallOneStep(:,k) = Pnew.';
    end
    ExecutionTime = toc(timerVal);
     
    CalkaP = trapz(Theta, PallOneStep, 2)/(2*pi);

    CalkaPdlaZetaOut = trapz(Theta,abs(PallOneStep(end,:).^2))/(2*pi);

    ElectronEfficiencyPerpendicullar = 1 - CalkaPdlaZetaOut;
    %plot(IN.RFfieldAxisZ,abs(CalkaP)) % debuging
    GyrotronEfficiency(step) = ElectronEfficiencyPerpendicullar*L.ElectronEfficiency;

     GyrotronOutputPower(step) = Icathode*(IN.Ucathode*ElectronEfficiencyPerpendicullar*L.ElectronEfficiency);
  
    if ( GyrotronOutputPower(step) < 0 ), GyrotronOutputPower(step) = 0; end 
   
    str=sprintf('t=%.2f [ns], CalkaPzetaOut=%f [a.u.], OutputPower=%.1f [kW], RK4 time=%.1f [s], Efi=%0.2f',t,abs(CalkaPdlaZetaOut),GyrotronOutputPower(step),ExecutionTime, GyrotronEfficiency(step));disp(str);

       
    step = step+1;
  
    tau = linspace(IN.TimeSpan(step-1),IN.TimeSpan(step),3)*(L.BetaPerpendicular^4/(8*L.BetaParallel^2))*OmegaCyclotron;
        
    timerVal = tic;
    NewfieldRaw = TimeDependentEq1(AxisZeta, field, tau, CalkaP, delta, kZend, IcathodeNormalized);
    ExecutionTime = toc(timerVal);
    str=sprintf('t=%.2f [ns], fmax=%f [a.u.], Pdepe time=%.1f [s]',t,fmax, ExecutionTime);disp(str);
    Field(:,step)= NewfieldRaw;

end

%% Zapamietanie wynikow
OUT.CharacteristicValue = L.CharacteristicValue;
OUT.RFfieldInTime = Field;
%OUT.ElectronMomentumTime = Pall;  % commented out to save RAM
OUT.ElectronBeamRadius = L.ElectronBeamRadius;
OUT.GyrotronEfficiency = GyrotronEfficiency;
OUT.GyrotronOutputPower = GyrotronOutputPower;
OUT.Icathode = Icathode;
OUT.TimeSpan = IN.TimeSpan;
OUT.Ucathode = IN.Ucathode;
OUT.Solver = solver;
%% Plot turned off for function use
%plot(t, GyrotronEfficiency);
%plot(t, Icathode)

end

