   function [tspan, yout] = ode4(tspan,y0, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1)
   %   [tspan, yout] = ode4(tspan,y0, resztaParametrowDlaWyliczeniaKroku)
   %    Matlabowy ode45 ma takie parametry:
   %    function varargout = ode45(ode,tspan,y0,options,varargin)
   % ODE4  Classical Runge-Kutta ODE solver.
   %   yout = ODE4(F,t0,h,tfinal,y0) uses the classical
   %   Runge-Kutta method with fixed step size h on the interval
   %      t0 <= t <= tfinal
   %   to solve
   %      dy/dt = F(t,y)
   %   with y(t0) = y0.

   %   Copyright 2014 - 2015 The MathWorks, Inc.
   
      y = y0;
      yout = y;
      h = (tspan(2)-tspan(1)); % wyliczam krok - zakladam ze jest staly
      timePoints = length(tspan);
      t = tspan;
      
      for i = 1:1:(timePoints-1)
         s1 = ode(t(i),y, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1);
         s2 = ode(t(i)+h/2, y+h*s1/2, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1);
         s3 = ode(t(i)+h/2, y+h*s2/2, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1);
         s4 = ode(t(i)+h, y+h*s3, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1);
         y = y + h*(s1 + 2*s2 + 2*s3 + s4)/6;
         yout = [yout; y]; %#ok<AGROW>
      end
   end
   
   function [ dPdZeta ] = ode( Zeta, P, ReRFfield, ImagRFfield, RFfieldZeta, Delta, n , F, S1)
        %EQELECTRONTRANSVERSEMOMENTUM Obliczamy moment elektronów - rownanie
        %różniczkowe
        %   Self consistent approach
        %RFfield = complex(ReRFfield, ImagRFfield);

        %RFfieldInterpolated = interp1(RFfieldZeta, RFfield, Zeta);

        % S1 gridded interpolant - szybciej niz interp
        RFfieldInterpolated = S1(Zeta);
        %Zeta
        %P
        
        dPdZeta = 1i* RFfieldInterpolated*P^(n-1)*F - 1i* (P/n) * (Delta + abs(P)^2 - 1);

    end      
      
%       for t = t0 : h : tfinal-h
%          s1 = ode(t,y);
%          s2 = ode(t+h/2, y+h*s1/2);
%          s3 = ode(t+h/2, y+h*s2/2);
%          s4 = ode(t+h, y+h*s3);
%          y = y + h*(s1 + 2*s2 + 2*s3 + s4)/6;
%          yout = [yout; y]; %#ok<AGROW>
%       end
